  <script defer
    src="/t.js"
    data-website-id="49cbb7c0-9b2a-45cc-8dab-95a8458cce48"
    data-host-url="https://umami.debruyn.dev"
    data-do-not-track="true">
  </script>
  <script type="text/javascript">
    (() => {
      const name = 'outbound-link-click';
      document.querySelectorAll('a').forEach(a => {
        if (a.host !== window.location.host && !a.getAttribute('data-umami-event')) {
          a.setAttribute('data-umami-event', name);
          a.setAttribute('data-umami-event-url', a.href);
        }
      });
    })();
  </script>